import ConfigParser
import re


class ConfigException(Exception):
    '''Config exception'''
    def __init__(self, message):
        Exception.__init__(self, message)
        


def checkSection(sec, valid):
    for k in valid:
        try:
            sec.remove(k)
        except:
            # not all valid keys need to be in the section
            pass
    return sec


class Config(object):
    def __init__(self, file):
        config = ConfigParser.SafeConfigParser()
        config.read(file)

        if not config.has_section('Mapping'):
            raise ConfigException('At least one mapping must be present in the Mapping section.')
        self.mapping = config.items('Mapping', True)

        check = checkSection(config.sections(), ['Mapping','Connection','Servers','Logging'])
        if check:
            raise ConfigException('There are unknown sections in the configuration file: %s' % str(check))

        self.connection = None
        if config.has_section('Connection'):
            self.connection = dict(config.items('Connection'))
        check = checkSection(self.connection.keys(), ['serverpath','sslkey_file','sslkey_pass','bidirectional','workers','force_source'])
        if check:
            raise ConfigException('There are unknown options in the Connection section: %s' % str(check))

        self.servers = {}
        if config.has_section('Servers'):
            self.servers = dict(config.items('Servers'))
            
        self.bidirectional = self.connection.get('bidirectional', True)
        self.workers = int(self.connection.get('workers', '4'))

        # Verify the destination servers.
        r = re.compile(r'^(http(s)?|file)://')
        for x in self.mapping:
            match = r.match(x[1] % self.servers)
            if match is None:
                raise ConfigException(x[0] + ': The destination server must be specified by its URL.')
            elif match.group(1) == 'http':
                raise ConfigException(x[0] + ': The remote destination server must use SSL.')
        
        self.config = config
